@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,10 +21,6 @@
 #ifndef ROCRAND_DISCRETE_H_
 #define ROCRAND_DISCRETE_H_
 
-#ifndef FQUALIFIERS
-#define FQUALIFIERS __forceinline__ __device__
-#endif // FQUALIFIERS
-
 #include <math.h>
 
 #include "rocrand/rocrand_lfsr113.h"
@@ -43,8 +39,6 @@
 #include "rocrand/rocrand_xorwow.h"
 
 #include "rocrand/rocrand_discrete_types.h"
-#include "rocrand/rocrand_normal.h"
-#include "rocrand/rocrand_uniform.h"
 
 // Alias method
 //
@@ -57,20 +51,31 @@
 namespace rocrand_device {
 namespace detail {
 
-FQUALIFIERS unsigned int discrete_alias(const double x, const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_alias(const double       x,
+                   const unsigned int size,
+                   const unsigned int offset,
+                   const unsigned int* __restrict__ alias,
+                   const double* __restrict__ probability)
 {
     // Calculate value using Alias table
 
     // x is [0, 1)
-    const double nx = dis.size * x;
-    const double fnx = floor(nx);
-    const double y = nx - fnx;
+    const double       nx  = size * x;
+    const double       fnx = floor(nx);
+    const double       y   = nx - fnx;
     const unsigned int i   = static_cast<unsigned int>(fnx);
-    return dis.offset + (y < dis.probability[i] ? i : dis.alias[i]);
+    return offset + (y < probability[i] ? i : alias[i]);
 }
 
-FQUALIFIERS
-unsigned int discrete_alias(const unsigned int r, const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_alias(const double x, const rocrand_discrete_distribution_st& dis)
+{
+    return discrete_alias(x, dis.size, dis.offset, dis.alias, dis.probability);
+}
+
+__forceinline__ __device__ __host__ unsigned int
+    discrete_alias(const unsigned int r, const rocrand_discrete_distribution_st& dis)
 {
     constexpr double inv_double_32 = ROCRAND_2POW32_INV_DOUBLE;
     const double x = r * inv_double_32;
@@ -78,33 +83,36 @@ unsigned int discrete_alias(const unsigned int r, const rocrand_discrete_distrib
 }
 
 // To prevent ambiguity compile error when compiler is facing the type "unsigned long"!!!
-FQUALIFIERS unsigned int discrete_alias(const unsigned long                     r,
-                                        const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_alias(const unsigned long r, const rocrand_discrete_distribution_st& dis)
 {
     constexpr double inv_double_32 = ROCRAND_2POW32_INV_DOUBLE;
     const double x = r * inv_double_32;
     return discrete_alias(x, dis);
 }
 
-FQUALIFIERS unsigned int discrete_alias(const unsigned long long int            r,
-                                        const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_alias(const unsigned long long int r, const rocrand_discrete_distribution_st& dis)
 {
     constexpr double inv_double_64 = ROCRAND_2POW64_INV_DOUBLE;
     const double x = r * inv_double_64;
     return discrete_alias(x, dis);
 }
 
-FQUALIFIERS unsigned int discrete_cdf(const double x, const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int discrete_cdf(const double       x,
+                                                              const unsigned int size,
+                                                              const unsigned int offset,
+                                                              const double* __restrict__ cdf)
 {
     // Calculate value using binary search in CDF
 
     unsigned int min = 0;
-    unsigned int max = dis.size - 1;
+    unsigned int max = size - 1;
     do
     {
         const unsigned int center = (min + max) / 2;
-        const double p = dis.cdf[center];
-        if (x > p)
+        const double       p      = cdf[center];
+        if(x > p)
         {
             min = center + 1;
         }
@@ -113,13 +121,19 @@ FQUALIFIERS unsigned int discrete_cdf(const double x, const rocrand_discrete_dis
             max = center;
         }
     }
-    while (min != max);
+    while(min != max);
 
-    return dis.offset + min;
+    return offset + min;
 }
 
-FQUALIFIERS
-unsigned int discrete_cdf(const unsigned int r, const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_cdf(const double x, const rocrand_discrete_distribution_st& dis)
+{
+    return discrete_cdf(x, dis.size, dis.offset, dis.cdf);
+}
+
+__forceinline__ __device__ __host__ unsigned int
+    discrete_cdf(const unsigned int r, const rocrand_discrete_distribution_st& dis)
 {
     constexpr double inv_double_32 = ROCRAND_2POW32_INV_DOUBLE;
     const double x = r * inv_double_32;
@@ -127,16 +141,16 @@ unsigned int discrete_cdf(const unsigned int r, const rocrand_discrete_distribut
 }
 
 // To prevent ambiguity compile error when compiler is facing the type "unsigned long"!!!
-FQUALIFIERS unsigned int discrete_cdf(const unsigned long                     r,
-                                      const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_cdf(const unsigned long r, const rocrand_discrete_distribution_st& dis)
 {
     constexpr double inv_double_32 = ROCRAND_2POW32_INV_DOUBLE;
     const double x = r * inv_double_32;
     return discrete_cdf(x, dis);
 }
 
-FQUALIFIERS unsigned int discrete_cdf(const unsigned long long int            r,
-                                      const rocrand_discrete_distribution_st& dis)
+__forceinline__ __device__ __host__ unsigned int
+    discrete_cdf(const unsigned long long int r, const rocrand_discrete_distribution_st& dis)
 {
     constexpr double inv_double_64 = ROCRAND_2POW64_INV_DOUBLE;
     const double x = r * inv_double_64;
@@ -163,8 +177,9 @@ FQUALIFIERS unsigned int discrete_cdf(const unsigned long long int            r,
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_philox4x32_10 * state, const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_philox4x32_10*        state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
 }
@@ -181,8 +196,8 @@ unsigned int rocrand_discrete(rocrand_state_philox4x32_10 * state, const rocrand
  *
  * \return Four <tt>unsigned int</tt> values distributed according to \p discrete_distribution as \p uint4
  */
-FQUALIFIERS
-uint4 rocrand_discrete4(rocrand_state_philox4x32_10 * state, const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ uint4 rocrand_discrete4(
+    rocrand_state_philox4x32_10* state, const rocrand_discrete_distribution discrete_distribution)
 {
     const uint4 u4 = rocrand4(state);
     return uint4 {
@@ -205,8 +220,9 @@ uint4 rocrand_discrete4(rocrand_state_philox4x32_10 * state, const rocrand_discr
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_mrg31k3p*             state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_mrg31k3p*             state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
 }
@@ -223,8 +239,9 @@ FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_mrg31k3p*             st
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_mrg32k3a * state, const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_mrg32k3a*             state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
 }
@@ -241,8 +258,9 @@ unsigned int rocrand_discrete(rocrand_state_mrg32k3a * state, const rocrand_disc
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_xorwow * state, const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_xorwow*               state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_alias(rocrand(state), *discrete_distribution);
 }
@@ -259,8 +277,9 @@ unsigned int rocrand_discrete(rocrand_state_xorwow * state, const rocrand_discre
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_mtgp32 * state, const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ unsigned int
+    rocrand_discrete(rocrand_state_mtgp32*               state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -277,8 +296,9 @@ unsigned int rocrand_discrete(rocrand_state_mtgp32 * state, const rocrand_discre
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_sobol32 * state, const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_sobol32*              state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -295,9 +315,9 @@ unsigned int rocrand_discrete(rocrand_state_sobol32 * state, const rocrand_discr
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_scrambled_sobol32*    state,
-                              const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_scrambled_sobol32*    state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -314,8 +334,9 @@ unsigned int rocrand_discrete(rocrand_state_scrambled_sobol32*    state,
  *
  * \return <tt>unsigned long long int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_sobol64*              state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_sobol64*              state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -332,8 +353,9 @@ FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_sobol64*              st
  *
  * \return <tt>unsigned long long int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_scrambled_sobol64*    state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_scrambled_sobol64*    state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -350,9 +372,9 @@ FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_scrambled_sobol64*    st
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS
-unsigned int rocrand_discrete(rocrand_state_lfsr113*              state,
-                              const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_lfsr113*              state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -369,8 +391,9 @@ unsigned int rocrand_discrete(rocrand_state_lfsr113*              state,
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry2x32_20*      state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_threefry2x32_20*      state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -387,8 +410,9 @@ FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry2x32_20*      st
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry2x64_20*      state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_threefry2x64_20*      state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -405,8 +429,9 @@ FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry2x64_20*      st
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry4x32_20*      state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_threefry4x32_20*      state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
@@ -423,8 +448,9 @@ FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry4x32_20*      st
  *
  * \return <tt>unsigned int</tt> value distributed according to \p discrete_distribution
  */
-FQUALIFIERS unsigned int rocrand_discrete(rocrand_state_threefry4x64_20*      state,
-                                          const rocrand_discrete_distribution discrete_distribution)
+__forceinline__ __device__ __host__ unsigned int
+    rocrand_discrete(rocrand_state_threefry4x64_20*      state,
+                     const rocrand_discrete_distribution discrete_distribution)
 {
     return rocrand_device::detail::discrete_cdf(rocrand(state), *discrete_distribution);
 }
